// ****************************************************************************
// Projekt      : Inbetriebname uP Board
// Prozessor    : PIC 18F4520 @ 40Mhz
// Compiler     : MPLAB XC8
// Autor        : Stefan Gluekler
// Datum        : 2018.12.31
// Version      : V3.0
// Beschreibung : Inbetriebname-Programm des uP Boards VFEI V2012 / V2013
//
// ****************************************************************************

// -- I N C L U D E S ---------------------------------------------------------

#include <xc.h>             // Include for the XC8 compiler
#include <stdbool.h>        // Include for data type 'bool' and keywords 'true' and 'false'

#include <string.h>         // Include for functions strcpy,strlen,...
#include <stdio.h>          // Include for functions sprintf,...

#include "plib/timers.h"    // Include for functions OpenTimer1,OpenTimer2,...
#include "plib/spi.h"       // Include for functions OpenSPI,...
#include "plib/pwm.h"       // Include for functions OpenPWM1,...
#include "plib/usart.h"     // Include for functions OpenUSART,...
#include "plib/adc.h"       // Include for functions OpenADC,...

#include "LCD_Module_DOGM163S-A.h"  // LCD Display VFEI Pic Board

// -- C O N F I G U R A T I O N   B I T S -------------------------------------
//--> Configuration_Bits.c

// -- D E F I N E S -----------------------------------------------------------

#define _XTAL_FREQ 40000000     // Crystal oscillator frequencies of 10Mhz VFEI-PIC-Board 2012 / Required by __delay_xx()

#define TASTER_LINKS    PORTBbits.RB0
#define TASTER_OBEN     PORTBbits.RB1
#define TASTER_UNTEN    PORTBbits.RB2
#define TASTER_RECHTS   PORTBbits.RB3
#define TASTER_MITTE    PORTBbits.RB4

#define LED_GRUEN1      LATAbits.LATA2
#define LED_GRUEN2      LATAbits.LATA3
#define LED_GELB        LATAbits.LATA4
#define LED_ROT         LATAbits.LATA5

#define VERSION_STRING  "V3.0 2018-12-31"

// -- G L O B A L E  V A R I A B L E N ----------------------------------------

unsigned int timer1_zaehler = 0;    // Hilfs-Variable fuer Uhrenquarz

                                                    // Bit  76543210
const unsigned char SiebenSegmentMappingArray[16] = { 0b00111111,   // 0
                                                      0b00000110,   // 1
                                                      0b01011011,   // 2
                                                      0b01001111,   // 3
                                                      0b01100110,   // 4
                                                      0b01101101,   // 5
                                                      0b01111101,   // 6
                                                      0b00000111,   // 7
                                                      0b01111111,   // 8
                                                      0b01101111,   // 9
                                                      0b01110111,   // A
                                                      0b01111100,   // b
                                                      0b00111001,   // C
                                                      0b01011110,   // d
                                                      0b01111001,   // E
                                                      0b01110001 }; // F

// -- F U K T I O N S  P R O T O T Y P E N ------------------------------------
void PortInit(void);
void InitRS232(void);
void DelayMs(unsigned int delayms);
void WriteConstString1USART(const char *data);
void WriteString1USART ( char *data);
void Menu_LEDs_farbig(void);
void Menu_7Segment(void);
void Menu_LEDs_blau(void);
void Menu_LCD_Anzeige(void);
void Menu_LCD_Beleuchtung(void);
void Menu_Kippschalter(void);
void Menu_Drucktasten(void);
void Menu_Potentiometer(void);
void Menu_UhrenQuarz(void);
void Menu_USB_Senden(void);
void Menu_USB_Empfangen(void);
void Menu_RS232_Senden(void);
void Menu_RS232_Empfangen(void);
void Menu_RS232_LoopBack(void);
void Menu_IOBox_Out(void);
void Menu_IOBox_In(void);

void timer1_isr(void);

// -- H A U P T P R O G R A M M -----------------------------------------------

void main(void)
{
  unsigned char aktuellerMenuPunkt = 1;

  // Init Ports -------------------------------------------------------------------------
  PortInit();

  // Init LCD Module --------------------------------------------------------------------
  LCD_Module_Init();
  LCD_Module_SetContrast(10);
  LCD_Module_SetBacklight(100);
          
  // Init RS232 -------------------------------------------------------------------------
  // InitRS232(); // wird nur bei Bedarf initialisiert

  // Init ADC ---------------------------------------------------------------------------
  OpenADC(ADC_FOSC_64 & ADC_RIGHT_JUST & ADC_12_TAD,
          ADC_CH0 & ADC_INT_OFF,
          ADC_REF_VDD_VDD & ADC_REF_VDD_VSS);

  // Init Timer 1 for external Quarz ----------------------------------------------------
  // Interrupt, 8Bit, Extern source, 1:1 Prescalar, Extern sync on, Oscilator on
  OpenTimer1(TIMER_INT_ON & T1_8BIT_RW & T1_SOURCE_PINOSC & T1_PS_1_1 & T1_SYNC_EXT_ON & T1_OSC1EN_ON,
             TIMER_GATE_OFF );
  // Timer overflow auf 32768
  WriteTimer1(0x8000);

  // Init Interrupts
  RCONbits.IPEN    = 1; // enable priority interrupt
  T1CONbits.TMR1ON = 0; // Turn timer 1 off
  IPR1bits.TMR1IP  = 1; // Set Timer1 Interrupt to high priority
  PIE1bits.TMR1IE  = 1; // Interrupt when Timer1 overflows
  INTCONbits.GIEH  = 1; // Enable Global High Priority Interrupts
  
  // Start ------------------------------------------------------------------------------
  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("VFEI PIC Board");
  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Inbetriebnahme");
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString(VERSION_STRING);
  DelayMs(2000);
  LCD_Module_ClearLine(2);
  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Microchip");
  LCD_Module_ClearLine(3);
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("PIC18F45K22");
  DelayMs(2000);
  LCD_Module_ClearLine(2);
  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Alle Jumper auf");
  LCD_Module_ClearLine(3);
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("'Board' setzen!");
  DelayMs(2000);

  // Kontrollieren, ob Taste "Oben" schon gedrueckt ist
  if (TASTER_OBEN == 0)
  {
    LCD_Module_ClearAll();
    LCD_Module_SetCursorPosition(1,1);
    LCD_Module_WriteString("ACHTUNG:");
    LCD_Module_SetCursorPosition(1,2);
    LCD_Module_WriteString("Taste 'Oben'");
    LCD_Module_SetCursorPosition(1,3);
    LCD_Module_WriteString("ist gedrueckt!");
    DelayMs(2000);
    while (TASTER_OBEN == 0) ;
  }

  // Kontrollieren, ob Taste "Unten" schon gedrueckt ist
  if (TASTER_UNTEN == 0)
  {
    LCD_Module_ClearAll();
    LCD_Module_SetCursorPosition(1,1);
    LCD_Module_WriteString("ACHTUNG:");
    LCD_Module_SetCursorPosition(1,2);
    LCD_Module_WriteString("Taste 'Unten'");
    LCD_Module_SetCursorPosition(1,3);
    LCD_Module_WriteString("ist gedrueckt!");
    DelayMs(2000);
    while (TASTER_UNTEN == 0) ;
  }

  // Kontrollieren, ob Taste "Oben" schon gedrueckt ist
  if (TASTER_MITTE == 0)
  {
    LCD_Module_ClearAll();
    LCD_Module_SetCursorPosition(1,1);
    LCD_Module_WriteString("ACHTUNG:");
    LCD_Module_SetCursorPosition(1,2);
    LCD_Module_WriteString("Taste 'Mitte'");
    LCD_Module_SetCursorPosition(1,3);
    LCD_Module_WriteString("ist gedrueckt!");
    DelayMs(2000);
    while (TASTER_MITTE == 0) ;
  }
    
  while(1) // Endlosschleife
  {
    bool menuAusgewaehltFlag = false;
    LCD_Module_ClearAll();
    LCD_Module_SetCursorPosition(1,1);
    LCD_Module_WriteString("VFEI PIC Board");
    LCD_Module_SetCursorPosition(1,2);
    LCD_Module_WriteString("Waehle aus:");
    do
    {
      bool neuerMenuPunktFlag = false;
      LCD_Module_ClearLine(3);
      LCD_Module_SetCursorPosition(1,3);
      switch(aktuellerMenuPunkt)
      {
        case 1:
          LCD_Module_SetCursorPosition(1,3);
          LCD_Module_WriteString("1. LED's farbig");
        break;
        case 2:
          LCD_Module_WriteString("2. 7-Segment");
        break;
        case 3:
          LCD_Module_WriteString("3. Blaue LED's");
        break;
        case 4:
          LCD_Module_WriteString("4. LCD Anzeige");
        break;
        case 5:
          LCD_Module_WriteString("5. LCD Beleucht.");
        break;
        case 6:
          LCD_Module_WriteString("6. Kippschalter");
        break;
        case 7:
          LCD_Module_WriteString("7. Drucktasten");
        break;
        case 8:
          LCD_Module_WriteString("8. Potentiometer");
        break;
        case 9:
          LCD_Module_WriteString("9. Uhren-Quarz");
        break;
        case 10:
          LCD_Module_WriteString("10.USB senden");
        break;
        case 11:
          LCD_Module_WriteString("11.USB empfangen");
        break;
        case 12:
          LCD_Module_WriteString("12.RS232 senden");
        break;
        case 13:
          LCD_Module_WriteString("13.RS232 empfan.");
        break;
        case 14:
          LCD_Module_WriteString("14.RS232 LoopBck");
        break;
        case 15:
          LCD_Module_WriteString("15.IO-Box LED");
        break;
        case 16:
          LCD_Module_WriteString("16.IO-Box Switch");
        break;
      } // switch()
      do
      {
        if (TASTER_OBEN == 0)
        {
          if (aktuellerMenuPunkt > 1)
            aktuellerMenuPunkt--;
          else
            aktuellerMenuPunkt = 16;
          neuerMenuPunktFlag = true;
          LCD_Module_ClearLine(3);
          while (TASTER_OBEN == 0) ;  // Warten, solange Taste gedrckt bleibt
        }
        if (TASTER_UNTEN == 0)
        {
          if (aktuellerMenuPunkt < 16)
            aktuellerMenuPunkt++;
          else
            aktuellerMenuPunkt = 1;
          neuerMenuPunktFlag = true;
          LCD_Module_ClearLine(3);
          while (TASTER_UNTEN == 0) ;  // Warten, solange Taste gedrckt bleibt
        }
        if (TASTER_MITTE == 0)
        {
          neuerMenuPunktFlag  = true;
          menuAusgewaehltFlag = true;
          LCD_Module_ClearLine(1);
          LCD_Module_ClearLine(2);
          LCD_Module_SetCursorPosition(1,1);
          LCD_Module_WriteString("Programm");
          LCD_Module_SetCursorPosition(1,2);
          LCD_Module_WriteString("ausgewaehlt:");
          while (TASTER_MITTE == 0) ;  // Warten, solange Taste gedrckt bleibt
        }  
      } while (neuerMenuPunktFlag == false);
    }  while(menuAusgewaehltFlag == false);

    switch(aktuellerMenuPunkt)
    {
      case 1:
        Menu_LEDs_farbig();
      break;
      case 2:
        Menu_7Segment();
      break;
      case 3:
        Menu_LEDs_blau();
      break;
      case 4:
        Menu_LCD_Anzeige();
      break;
      case 5:
        Menu_LCD_Beleuchtung();
      break;
      case 6:
        Menu_Kippschalter();
      break;
      case 7:
        Menu_Drucktasten();
      break;
      case 8:
        Menu_Potentiometer();
      break;
      case 9:
        Menu_UhrenQuarz();
      break;
      case 10:
        Menu_USB_Senden();
      break;
      case 11:
        Menu_USB_Empfangen();
      break;
      case 12:
        Menu_RS232_Senden();
      break;
      case 13:
        Menu_RS232_Empfangen();
      break;
      case 14:
        Menu_RS232_LoopBack();
      break;
      case 15:
        Menu_IOBox_Out();
      break;
      case 16:
        Menu_IOBox_In();
      break;
    } // switch()

  } // while(1)

} // main()

// Funktion:     PortInit()
// Beschreibung: Initialisierung der IO-Ports
// Parameter:    -
// Rueckgabe:    -
void PortInit(void)
{
  // *** PortA[0-5]: Initialisierung *******************************************
  // PortA[0]: Initialisierung als analoger Eingang fr Potentiometer (unten)
  TRISAbits.TRISA0 = 1; ANSELAbits.ANSA0 = 1;    // PortA[0]: Analog-Input-Port

  // PortA[1]: Initialisierung als analoger Eingang fr Potentiometer (oben)
  TRISAbits.TRISA1 = 1; ANSELAbits.ANSA1 = 1;    // PortA[1]: Analog-Input-Port

  // PortA[2]: Initialisierung als digitaler Output fr LED blau
  TRISAbits.TRISA2 = 0; ANSELAbits.ANSA2 = 0;    // PortA[2]: Output-Port
  LATAbits.LATA2   = 0; // LED aus

  // PortA[3]: Initialisierung als digitaler Output fr LED gruen
  TRISAbits.TRISA3 = 0; ANSELAbits.ANSA3 = 0;    // PortA[3]: Output-Port
  LATAbits.LATA3   = 0; // LED aus

  // PortA[4]: Initialisierung als digitaler Output fr LED gelb
  TRISAbits.TRISA4 = 0;                          // PortA[4]: Output-Port
  LATAbits.LATA4   = 0; // LED aus

  // PortA[5]: Initialisierung als digitaler Output fr LED rot
  TRISAbits.TRISA5 = 0; ANSELAbits.ANSA5 = 0;    // PortA[5]: Output-Port
  LATAbits.LATA5   = 0; // LED aus

  // PortA[6]:  10MHz Quarz
  // PortA[7]:  10MHz Quarz

  // *** PortB[0-5]: Initialisierung *******************************************
  // PortB[0]: Initialisierung als digitaler Input fr Taster links
  TRISBbits.TRISB0 = 1; ANSELBbits.ANSB0 = 0;    // PortB[0]: Input-Port

  // PortB[1]: Initialisierung als digitaler Input fr Taster oben
  TRISBbits.TRISB1 = 1; ANSELBbits.ANSB1 = 0;    // PortB[1]: Input-Port

  // PortB[2]: Initialisierung als digitaler Input fr Taster unten
  TRISBbits.TRISB2 = 1; ANSELBbits.ANSB2 = 0;    // PortB[2]: Input-Port

  // PortB[3]: Initialisierung als digitaler Input fr Taster rechts
  TRISBbits.TRISB3 = 1; ANSELBbits.ANSB3 = 0;    // PortB[3]: Input-Port

  // PortB[4]: Initialisierung als digitaler Input fr Taster mitte
  TRISBbits.TRISB4 = 1; ANSELBbits.ANSB4 = 0;    // PortB[4]: Input-Port

  // PortB[5]: Initialisierung als digitaler Output fr LCD _Cmd/Data
  TRISBbits.TRISB5 = 0; ANSELBbits.ANSB5 = 0;    // PortB[4]: Output-Port
  LATBbits.LATB0 = 0;

  // PortB[6]:  PICKit ? PGC
  // PortB[7]:  PICKit ? PGD

  // *** PortC[0-7]: Initialisierung *******************************************
  // PortC[0]: Initialisierung als digitaler Input fr 32.768 kHz Uhren-Quarz
  TRISCbits.TRISC0 = 1;                          // PortC[0]: Input-Port

  // PortC[1]: Initialisierung als digitaler Input fr 32.768 kHz Uhren-Quarz
  TRISCbits.TRISC1 = 1;                          // PortC[1]: Input-Port

  // PortC[2]: Initialisierung als digitaler Output fr PWM LCD Beleuchtung
  TRISCbits.TRISC2 = 0; ANSELCbits.ANSC2 = 0;    // PortC[2]: Output-Port
  LATCbits.LATC2 = 0;

  // PortC[3]: Initialisierung als digitaler Output fr LCD SPI Clock
  TRISCbits.TRISC3 = 0; ANSELCbits.ANSC3 = 0;    // PortC[3]: Output-Port
  LATCbits.LATC3 = 0;

  // PortC[4]: Initialisierung als digitaler Output fr LCD SPI Chip-Select
  TRISCbits.TRISC4 = 0; ANSELCbits.ANSC4 = 0;    // PortC[4]: Output-Port
  LATCbits.LATC4 = 0;

  // PortC[5]: Initialisierung als digitaler Output fr LCD SPI Data
  TRISCbits.TRISC5 = 0; ANSELCbits.ANSC5 = 0;    // PortC[5]: Output-Port
  LATCbits.LATC5 = 0;

  // PortC[6]: Initialisierung als digitaler Input fr RS232 TX (gemss Datenblatt!)
  TRISCbits.TRISC6 = 1; ANSELCbits.ANSC6 = 0;    // PortC[6]: Input-Port

  // PortC[7]: Initialisierung als digitaler Input fr RS232 RX (gemss Datenblatt!)
  TRISCbits.TRISC7 = 1; ANSELCbits.ANSC7 = 0;    // PortC[7]: Input-Port

  // *** PortD[0-7]: Initialisierung *******************************************
  // PortD[0-7]: Initialisierung als digitaler Output fr die 1 Digit Siebensegment-Anzeige
  TRISD  = 0xFF; // PortD[0-7]: Input-Port, bei Bedarf als Output
  LATD   = 0x00; // PortD[0-7]: LEDs aus
  ANSELD = 0x00; // keine Analogen Port-Pins

  // *** PortE[0-2]: Initialisierung *******************************************
  // PortE[0]: Initialisierung als digitaler Input fr Schalter RE0
  TRISEbits.TRISE0 = 1; ANSELEbits.ANSE0 = 0;    // PortE[0]: Input-Port

  // PortE[1]: Initialisierung als digitaler Input fr Schalter RE1
  TRISEbits.TRISE1 = 1; ANSELEbits.ANSE1 = 0;    // PortE[1]: Input-Port

  // PortE[2]: Initialisierung als digitaler Input fr Schalter RE2
  TRISEbits.TRISE2 = 1; ANSELEbits.ANSE2 = 0;    // PortE[2]: Input-Port
}

void InitRS232(void)
{
  // PortC[6]: Initialisierung als digitaler Input fr RS232 TX (gemss Datenblatt!)
  TRISCbits.TRISC6 = 1; ANSELCbits.ANSC6 = 0;    // PortC[6]: Input-Port

  // PortC[7]: Initialisierung als digitaler Input fr RS232 RX (gemss Datenblatt!)
  TRISCbits.TRISC7 = 1; ANSELCbits.ANSC7 = 0;    // PortC[7]: Input-Port

  // FOSC = 40MHz / USART_BRGH_LOW :
  // 2400 Baud  -> 255
  // 9600 Baud  -> 64
  // 19200 Baud -> 31
  // 57600 Baud -> 10
  Close1USART();
  Open1USART(USART_TX_INT_OFF & USART_RX_INT_OFF &
            USART_EIGHT_BIT & USART_CONT_RX & USART_ASYNCH_MODE &
            USART_BRGH_LOW, 255);  // 2400 Baud 8, N, 1
}

// Funktion:     DelayMs()
// Beschreibung: Wartet die angegebene Zeit in Milisekunden
// Parameter:    delayms - Zeit in Milisekunden
// Rueckgabe:    -
void DelayMs(unsigned int delayms)
{
  for (unsigned int i = 0 ; i < delayms ; i++)
  {
    __delay_us(1000);
  } // for()
}

// Funktion:     WriteConstString1USART()
// Beschreibung: Test-Programm fuer farbige LEDs
// Parameter:    Konstanter String
// Rueckgabe:    -
void WriteConstString1USART(const char *data)
{
  while ( *data != '\0' )
  { // Transmit a byte
    while ( Busy1USART( ) );
    putc1USART( *data );
    ++data;
  }
}

// Funktion:     WriteString1USART()
// Beschreibung: Test-Programm fuer farbige LEDs
// Parameter:    Char Array (String)
// Rueckgabe:    -
void WriteString1USART ( char *data)
{
  while ( *data != '\0' )
  { // Transmit a byte
    while ( Busy1USART( ) );
    putc1USART( *data );
    ++data;
  }
}


// Funktion:     Menu_LEDs_farbig()
// Beschreibung: Test-Programm fuer farbige LEDs
// Parameter:    -
// Rueckgabe:    -
void Menu_LEDs_farbig(void)
{
  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("1. LED's farbig");

  LED_GRUEN1 = 1;
  LCD_Module_ClearLine(2);
  LCD_Module_ClearLine(3);
  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Die blaue LED");
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("leuchtet");
  DelayMs(2000);
  LED_GRUEN1 = 0;

  LED_GRUEN2 = 1;
  LCD_Module_ClearLine(2);
  LCD_Module_ClearLine(3);
  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Die gruene LED");
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("leuchtet");
  DelayMs(2000);
  LED_GRUEN2 = 0;

  LED_GELB = 1;
  LCD_Module_ClearLine(2);
  LCD_Module_ClearLine(3);
  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Die gelbe LED");
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("leuchtet");
  DelayMs(2000);
  LED_GELB = 0;

  LED_ROT = 1;
  LCD_Module_ClearLine(2);
  LCD_Module_ClearLine(3);
  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Die rote LED");
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("leuchtet");
  DelayMs(2000);
  LED_ROT = 0;

  LCD_Module_ClearAll();
}  

// Funktion:     Menu_7Segment()
// Beschreibung: Test-Programm fuer 7-Segement-Anzeige
// Parameter:    -
// Rueckgabe:    -
void Menu_7Segment(void)
{
  unsigned char zahl;

  TRISD  = 0x00; // PortD[0-7]: Output-Port


  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("2. 7-Segment");

  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Zahlen 0 bis 9");
  for(zahl=0;zahl<10;zahl++)
  {
    LCD_Module_SetCursorPosition(1,3);
    LCD_Module_WriteFormated_Int(zahl,"-> Zahl %d");
    LATD = SiebenSegmentMappingArray[zahl];
    DelayMs(1000);
  } // for()

  LCD_Module_ClearLine(2);
  LCD_Module_ClearLine(3);
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("-> Dezimalpunkt");
  LATD = 0b10000000;
  DelayMs(2000);
  
  LATD = 0x00;
  LCD_Module_ClearAll();

  TRISD  = 0xFF; // PortD[0-7]: Input-Port
}

// Funktion:     Menu_LEDs_blau()
// Beschreibung: Test-Programm fuer die 8 gruene LEDs
// Parameter:    -
// Rueckgabe:    -
void Menu_LEDs_blau(void)
{
  unsigned char ledBit = 0x01;
  unsigned char zaehler;
  
  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("3. Blaue LED's");
  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteFormated_Int(zaehler,"LEDs RD0 bis RD7");

  TRISD  = 0x00; // PortD[0-7]: Output-Port

  for(zaehler=0;zaehler<8;zaehler++)
  {
    LCD_Module_SetCursorPosition(1,3);
    LCD_Module_WriteFormated_Int(zaehler,"-> LED RD%d");
    LATD = ledBit;
    DelayMs(1000);
    ledBit = ledBit << 1;
  } // for()

  LCD_Module_ClearLine(3);
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("-> Alle LEDs ein");
  LATD = 0xFF;
  DelayMs(1000);

  LCD_Module_ClearLine(3);
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("-> Alle LEDs aus");
  LATD = 0x00;
  DelayMs(1000);

  LATD = 0x00;
  LCD_Module_ClearAll();
  TRISD  = 0xFF; // PortD[0-7]: Input-Port
}

// Funktion:     Menu_LCD_Anzeige()
// Beschreibung: Test-Programm fuer die 8 gruene LEDs
// Parameter:    -
// Rueckgabe:    -
void Menu_LCD_Anzeige(void)
{
  unsigned char zeichen;
  
  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("4. LCD Anzeige");

  LCD_Module_SetCursorPosition(1,2);
  for (zeichen='A';zeichen<='Z';zeichen++)
  {
    LCD_Module_WriteChar(zeichen);
    DelayMs(300);
  } // for()  
  for (zeichen='0';zeichen<='5';zeichen++)
  {
    LCD_Module_WriteChar(zeichen);
    DelayMs(300);
  } // for()  
  LCD_Module_SetCursorPosition(1,2);
  for (zeichen='6';zeichen<='9';zeichen++)
  {
    LCD_Module_WriteChar(zeichen);
    DelayMs(300);
  } // for()  
  for (zeichen='A';zeichen<='Z';zeichen++)
  {
    LCD_Module_WriteChar(zeichen);
    DelayMs(300);
  } // for()  

  LCD_Module_ClearAll();
}

// Funktion:     Menu_LCD_Beleuchtung()
// Beschreibung: Test-Programm fuer die LCD Hintergrundbeleuchtung
// Parameter:    -
// Rueckgabe:    -
void Menu_LCD_Beleuchtung(void)
{
  unsigned char prozent;
  
  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("5. LCD Beleucht.");

  for(prozent=100;prozent>=1;prozent--)
  {
    LCD_Module_SetCursorPosition(1,3);
    LCD_Module_WriteFormated_UInt(prozent,"Beleuchtung:%3u%%");
    LCD_Module_SetBacklight(prozent);
    DelayMs(100);
  } // for()
  for(prozent=1;prozent<=100;prozent++)
  {
    LCD_Module_SetCursorPosition(1,3);
    LCD_Module_WriteFormated_UInt(prozent,"Beleuchtung:%3u%%");
    LCD_Module_SetBacklight(prozent);
    DelayMs(100);
  } // for()

  DelayMs(1000);
  LCD_Module_SetBacklight(100);
  LCD_Module_ClearAll();
}

// Funktion:     Menu_Kippschalter()
// Beschreibung: Test-Programm fuer die Kipschalter
// Parameter:    -
// Rueckgabe:    -
void Menu_Kippschalter(void)
{
  bool flag_RE0 = false , flag_RE1 = false , flag_RE2 = false;
  unsigned int counter_RE0 = 0 , counter_RE1 = 0 , counter_RE2 = 0;
  unsigned int durchlauf;
  
  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("6. Kippschalter");

  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("RE0  RE1  RE2");
  for(durchlauf=200;durchlauf>0;durchlauf--)
  {
    LCD_Module_SetCursorPosition(14,2);
    LCD_Module_WriteFormated_Int(durchlauf/10,"%3d");
    if (PORTEbits.RE0 == 1)
    {
      LCD_Module_SetCursorPosition(1,3);
      LCD_Module_WriteString("ein");
      if (flag_RE0 == false)
      {
        flag_RE0 = true;
        counter_RE0++;
        LCD_Module_SetCursorPosition(4,3);
        LCD_Module_WriteFormated_Int(counter_RE0%10,"%1d");
      }  
    }
    else
    {
      LCD_Module_SetCursorPosition(1,3);
      LCD_Module_WriteString("aus");
      if (flag_RE0 == true)
      {
        flag_RE0 = false;
        counter_RE0++;
        LCD_Module_SetCursorPosition(4,3);
        LCD_Module_WriteFormated_Int(counter_RE0%10,"%1d");
      }  
    } // if (RE0)
    if (PORTEbits.RE1 == 1)
    {
      LCD_Module_SetCursorPosition(6,3);
      LCD_Module_WriteString("ein");
      if (flag_RE1 == false)
      {
        flag_RE1 = true;
        counter_RE1++;
        LCD_Module_SetCursorPosition(9,3);
        LCD_Module_WriteFormated_Int(counter_RE1%10,"%1d");
      }  
    }
    else
    {
      LCD_Module_SetCursorPosition(6,3);
      LCD_Module_WriteString("aus");
      if (flag_RE1 == true)
      {
        flag_RE1 = false;
        counter_RE1++;
        LCD_Module_SetCursorPosition(9,3);
        LCD_Module_WriteFormated_Int(counter_RE1%10,"%1d");
      }  
    }// if (RE1)
    if (PORTEbits.RE2 == 1)
    {
      LCD_Module_SetCursorPosition(11,3);
      LCD_Module_WriteString("ein");
      if (flag_RE2 == false)
      {
        flag_RE2 = true;
        counter_RE2++;
        LCD_Module_SetCursorPosition(14,3);
        LCD_Module_WriteFormated_Int(counter_RE2%10,"%1d");
      }  
    }
    else
    {
      LCD_Module_SetCursorPosition(11,3);
      LCD_Module_WriteString("aus");
      if (flag_RE2 == true)
      {
        flag_RE2 = false;
        counter_RE2++;
        LCD_Module_SetCursorPosition(14,3);
        LCD_Module_WriteFormated_Int(counter_RE2%10,"%1d");
      }  
    }// if (RE2)
    DelayMs(100);
  } // for()

  LCD_Module_ClearAll();
}

// Funktion:     Menu_Drucktasten()
// Beschreibung: Test-Programm fuer die Drucktasten
// Parameter:    -
// Rueckgabe:    -
void Menu_Drucktasten(void)
{
  bool flag_RB0 = false , flag_RB1 = false , flag_RB2 = false , flag_RB3 = false , flag_RB4 = false;
  unsigned int counter_RB0 = 0 , counter_RB1 = 0 , counter_RB2 = 0 , counter_RB3 = 0 , counter_RB4 = 0;
  unsigned int durchlauf;

  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("7. Drucktasten");

  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Druecke Taste");

  for(durchlauf=200;durchlauf>0;durchlauf--)
  {
    LCD_Module_SetCursorPosition(14,2);
    LCD_Module_WriteFormated_Int(durchlauf/10,"%3d");
    if (PORTBbits.RB0 == 0) // Taste Links
    {
      if (flag_RB0 == false)
      {
        flag_RB0 = true;
        counter_RB0++;
      }  
    }
    else
    {
      if (flag_RB0 == true)
      {
        flag_RB0 = false;
      }  
    } // RB0
    if (PORTBbits.RB1 == 0) // Taste Oben
    {
      if (flag_RB1 == false)
      {
        flag_RB1 = true;
        counter_RB1++;
      }  
    }
    else
    {
      if (flag_RB1 == true)
      {
        flag_RB1 = false;
      }  
    } // RB1
    if (PORTBbits.RB2 == 0) // Taste Unten
    {
      if (flag_RB2 == false)
      {
        flag_RB2 = true;
        counter_RB2++;
      }  
    }
    else
    {
      if (flag_RB2 == true)
      {
        flag_RB2 = false;
      }  
    } // RB2
    if (PORTBbits.RB3 == 0) // Taste Rechts
    {
      if (flag_RB3 == false)
      {
        flag_RB3 = true;
        counter_RB3++;
      }  
    }
    else
    {
      if (flag_RB3 == true)
      {
        flag_RB3 = false;
      }  
    } // RB3
    if (PORTBbits.RB4 == 0) // Taste Mitte
    {
      if (flag_RB4 == false)
      {
        flag_RB4 = true;
        counter_RB4++;
      }  
    }
    else
    {
      if (flag_RB4 == true)
      {
        flag_RB4 = false;
      }  
    } // RB4
    if (flag_RB0 == false && flag_RB1 == false && flag_RB2 == false && flag_RB3 == false && flag_RB4 == false) // Keine Taste
    {
      LCD_Module_ClearLine(3);
    }
    else if (flag_RB0 == true  && flag_RB1 == false && flag_RB2 == false && flag_RB3 == false && flag_RB4 == false) // Taste RB0 Links
    {
      LCD_Module_SetCursorPosition(1,3);
      LCD_Module_WriteString("T. RB0 Links");
      LCD_Module_SetCursorPosition(15,3);
      LCD_Module_WriteFormated_Int(counter_RB0%100,"%2d");
    }
    else if (flag_RB0 == false && flag_RB1 == true  && flag_RB2 == false && flag_RB3 == false && flag_RB4 == false) // Taste RB1 Oben
    {
      LCD_Module_SetCursorPosition(1,3);
      LCD_Module_WriteString("T. RB1 Oben");
      LCD_Module_SetCursorPosition(15,3);
      LCD_Module_WriteFormated_Int(counter_RB1%100,"%2d");
    }
    else if (flag_RB0 == false && flag_RB1 == false && flag_RB2 == true  && flag_RB3 == false && flag_RB4 == false) // Taste RB2 Unten
    {
      LCD_Module_SetCursorPosition(1,3);
      LCD_Module_WriteString("T. RB2 Unten");
      LCD_Module_SetCursorPosition(15,3);
      LCD_Module_WriteFormated_Int(counter_RB2%100,"%2d");
    }
    else if (flag_RB0 == false && flag_RB1 == false && flag_RB2 == false && flag_RB3 == true  && flag_RB4 == false) // Taste RB3 Rechts
    {
      LCD_Module_SetCursorPosition(1,3);
      LCD_Module_WriteString("T. RB3 Rechts");
      LCD_Module_SetCursorPosition(15,3);
      LCD_Module_WriteFormated_Int(counter_RB3%100,"%2d");
    }
    else if (flag_RB0 == false && flag_RB1 == false && flag_RB2 == false && flag_RB3 == false && flag_RB4 == true ) // Taste RB4 Mitte
    {
      LCD_Module_SetCursorPosition(1,3);
      LCD_Module_WriteString("T. RB4 Mitte");
      LCD_Module_SetCursorPosition(15,3);
      LCD_Module_WriteFormated_Int(counter_RB4%100,"%2d");
    }
    else // Mehrere Tasten sind gedrueckt
    {
      LCD_Module_SetCursorPosition(1,3);
      LCD_Module_WriteString("Mehrere Tasten !");
      DelayMs(500);
    }  
    DelayMs(100);
  } // for()
  
  LCD_Module_ClearAll();
}

// Funktion:     Menu_Potentiometer()
// Beschreibung: Test-Programm fuer die Potentiometer
// Parameter:    -
// Rueckgabe:    -
void Menu_Potentiometer(void)
{
  unsigned int durchlauf;

  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("8. Potentiometer");

  for(durchlauf=200;durchlauf>0;durchlauf--)
  {
    unsigned int adWert;
    LCD_Module_SetCursorPosition(14,2);
    LCD_Module_WriteFormated_Int(durchlauf/10,"%3d");
    SetChanADC(ADC_CH0);
    ConvertADC();
    while(BusyADC());
    adWert = ReadADC();
    LCD_Module_SetCursorPosition(1,2);
    LCD_Module_WriteFormated_Int(adWert,"Poti RA0 %-4d");
    SetChanADC(ADC_CH1);
    ConvertADC();
    while(BusyADC());
    adWert = ReadADC();
    LCD_Module_SetCursorPosition(1,3);
    LCD_Module_WriteFormated_Int(adWert,"Poti RA1 %-4d");
    DelayMs(100);
  } // for()
    
  LCD_Module_ClearAll();
}

// Funktion:     Menu_UhrenQuarz()
// Beschreibung: Test-Programm fuer den Uhrenquarz
// Parameter:    -
// Rueckgabe:    -
void Menu_UhrenQuarz(void)
{
  unsigned int durchlauf;

  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("9. Uhren-Quarz");

  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Zeit in Sek.");

  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("Uhr 00:00");

  // LEDs Aus
  LED_GELB = 0;
  LED_ROT = 0;

  // Timer overflow auf 32768
  WriteTimer1(0x8000);
  T1CONbits.TMR1ON = 1; // Turn timer 1 on
  timer1_zaehler = 0;

  for(durchlauf=200;durchlauf>0;durchlauf--)
  {
    unsigned int zeit = timer1_zaehler;
    LCD_Module_SetCursorPosition(14,2);
    LCD_Module_WriteFormated_Int(durchlauf/10,"%3d");
    LCD_Module_SetCursorPosition(5,3);
    LCD_Module_WriteFormated_Int(zeit/60,"%02d");
    LCD_Module_SetCursorPosition(8,3);
    LCD_Module_WriteFormated_Int(zeit%60,"%02d");
    DelayMs(100);
  } // for()

  T1CONbits.TMR1ON = 0; // Turn timer 1 off

  // LEDs Aus
  LED_GELB = 0;
  LED_ROT = 0;
    
  LCD_Module_ClearAll();
}

// Funktion:     Menu_USB_Senden()
// Beschreibung: Test-Programm fuer Senden ueber USB
// Parameter:    -
// Rueckgabe:    -
void Menu_USB_Senden(void)
{
  unsigned int durchlauf;
  unsigned int textNr = 1;

  InitRS232();

  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("10.USB senden");

  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Jumper auf USB");
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("2400 Baud 8,N,1");
  DelayMs(3000);
  LCD_Module_ClearLine(2);
  LCD_Module_ClearLine(3);

  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Sende Text");

  WriteConstString1USART("\r\n");
  while(Busy1USART()) ;

  for(durchlauf=120;durchlauf>0;durchlauf--)
  {
    LCD_Module_SetCursorPosition(14,2);
    LCD_Module_WriteFormated_Int(durchlauf/10,"%3d");
    if ((durchlauf % 10) == 0)
    {
      char text[20];
      LCD_Module_ClearLine(3);
      LCD_Module_SetCursorPosition(1,3);
      switch(textNr)
      {
        case 1:
          strcpy(text,"Die ");
          textNr=2;
        break;
        case 2:
          strcpy(text,"Kommunikation ");
          textNr=3;
        break;
        case 3:
          strcpy(text,"ueber USB ");
          textNr=4;
        break;
        case 4:
          strcpy(text,"funktioniert!");
          textNr=1;
        break;
      } // switch()
      LCD_Module_WriteString(text);
      WriteString1USART(text);
      while(Busy1USART()) ;
      if (textNr == 1)
      {
        WriteConstString1USART("\r\n");
        while(Busy1USART()) ;
      }  
    } // if()
    DelayMs(100);
  } // for()

  LCD_Module_ClearAll();

  Close1USART();
}

// Funktion:     Menu_USB_Empfangen()
// Beschreibung: Test-Programm fuer Empfangen ueber USB
// Parameter:    -
// Rueckgabe:    -
void Menu_USB_Empfangen(void)
{
  char zeichen;
  unsigned int durchlauf;
  unsigned int position = 1;

  InitRS232();

  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("10.USB empfangen");

  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Jumper auf USB");
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("2400 Baud 8,N,1");
  DelayMs(3000);
  LCD_Module_ClearLine(2);
  LCD_Module_ClearLine(3);
  
  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Empfange Text");

  for(durchlauf=20000;durchlauf>0;durchlauf--)
  {
    LCD_Module_SetCursorPosition(14,2);
    LCD_Module_WriteFormated_Int(durchlauf/1000,"%3d");
    if (DataRdy1USART())
    {
      zeichen = getc1USART();
      LCD_Module_SetCursorPosition(position,3);
      LCD_Module_WriteChar(zeichen);
      Write1USART(zeichen);
      while(Busy1USART()) ;
      if (++position > 16)
        position = 1;
    }
    DelayMs(1);
  } // for()

  LCD_Module_ClearAll();

  Close1USART();
}
  
// Funktion:     Menu_RS232_Senden()
// Beschreibung: Test-Programm fuer Senden ueber RS232
// Parameter:    -
// Rueckgabe:    -
void Menu_RS232_Senden(void)
{
  unsigned int durchlauf;
  unsigned int textNr = 1;

  InitRS232();

  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("12.RS232 senden");

  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Jumper auf RS232");
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("2400 Baud 8,N,1");
  DelayMs(3000);
  LCD_Module_ClearLine(2);
  LCD_Module_ClearLine(3);

  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Sende Text");

  WriteConstString1USART("\r\n");
  while(Busy1USART()) ;

  for(durchlauf=120;durchlauf>0;durchlauf--)
  {
    LCD_Module_SetCursorPosition(14,2);
    LCD_Module_WriteFormated_Int(durchlauf/10,"%3d");
    if ((durchlauf % 10) == 0)
    {
      char text[20];
      LCD_Module_ClearLine(3);
      LCD_Module_SetCursorPosition(1,3);
      switch(textNr)
      {
        case 1:
          strcpy(text,"Die ");
          textNr=2;
        break;
        case 2:
          strcpy(text,"Kommunikation ");
          textNr=3;
        break;
        case 3:
          strcpy(text,"ueber RS232 ");
          textNr=4;
        break;
        case 4:
          strcpy(text,"funktioniert!");
          textNr=1;
        break;
      } // switch()
      LCD_Module_WriteString(text);
      WriteString1USART(text);
      while(Busy1USART()) ;
      if (textNr == 1)
      {
        WriteConstString1USART("\r\n");
        while(Busy1USART()) ;
      }  
    } // if()
    DelayMs(100);
  } // for()

  LCD_Module_ClearAll();

  Close1USART();
}

// Funktion:     Menu_RS232_Empfangen()
// Beschreibung: Test-Programm fuer Empfangen ueber RS232
// Parameter:    -
// Rueckgabe:    -
void Menu_RS232_Empfangen(void)
{
  unsigned int durchlauf;
  unsigned int position = 1;

  InitRS232();

  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("13.RS232 empfan.");

  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Jumper auf RS232");
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("2400 Baud 8,N,1");
  DelayMs(3000);
  LCD_Module_ClearLine(2);
  LCD_Module_ClearLine(3);
  
  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Empfange Text");

  for(durchlauf=20000;durchlauf>0;durchlauf--)
  {
    LCD_Module_SetCursorPosition(14,2);
    LCD_Module_WriteFormated_Int(durchlauf/1000,"%3d");
    if (DataRdy1USART())
    {
      char zeichen;
      zeichen = getc1USART();
      LCD_Module_SetCursorPosition(position,3);
      LCD_Module_WriteChar(zeichen);
      Write1USART(zeichen);
      while(Busy1USART()) ;
      if (++position > 16)
        position = 1;
    }
    DelayMs(1);
  } // for()

  LCD_Module_ClearAll();

  Close1USART();
}

// Funktion:     Menu_RS232_LoopBack()
// Beschreibung: Test-Programm fuer Senden und Empfangen ueber RS232 ohne PC
// Parameter:    -
// Rueckgabe:    -
void Menu_RS232_LoopBack(void)
{
  unsigned int durchlauf;
  unsigned int errorcnt;
  unsigned int position;
  char zeichen;
  unsigned int testchar[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

  InitRS232();

  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("14.RS232 LoopBck");

  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Jumper auf RS232");
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("LoopBack Stecker");
  DelayMs(3000);
  LCD_Module_ClearLine(2);
  LCD_Module_ClearLine(3);

  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Loopback-Test");

  position = 1;
  for(durchlauf=16;durchlauf>0;durchlauf--)
  {
    LCD_Module_SetCursorPosition(14,2);
    LCD_Module_WriteFormated_Int(durchlauf/2,"%3d");

    Write1USART(testchar[16-durchlauf]);    // Test-Zeichen senden...
    while(Busy1USART()) ;

    // Auf Zeichen warten...
    errorcnt = 0;
    while (!DataRdy1USART() && (errorcnt < 9))
    {
      ++errorcnt;
      DelayMs(1);
    }

    zeichen = getc1USART(); // Zeichen lesen

    // Gesendetes und Empfangenes Zeichen vergleichen...
    if (zeichen == testchar[16-durchlauf])
    {
      LCD_Module_SetCursorPosition(position,3);
      LCD_Module_WriteChar(zeichen);
    }
    else
    {
      LCD_Module_SetCursorPosition(position,3);
      LCD_Module_WriteChar('x');
    }

    ++position;

    DelayMs(500);
  }
  
  LCD_Module_ClearAll();

  Close1USART();
}

// Funktion:     Menu_IOBox_Out()
// Beschreibung: Test-Programm IO-Box LEDs
// Parameter:    -
// Rueckgabe:    -
void Menu_IOBox_Out(void)
{
  unsigned char ledBit = 0x01;
  unsigned char zaehler;

  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1,1);
  LCD_Module_WriteString("15. IO-Box LEDs");

  LCD_Module_SetCursorPosition(1,2);
  LCD_Module_WriteString("Port D Jumper");
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("auf Pos. Header");
  DelayMs(3000);
  LCD_Module_ClearLine(2);
  LCD_Module_ClearLine(3);

  TRISD  = 0x00; // PortD[0-7]: Output-Port

  LCD_Module_SetCursorPosition(1,2);
  for(zaehler=0;zaehler<8;zaehler++)
  {
    LCD_Module_SetCursorPosition(1,3);
    LCD_Module_WriteFormated_Int(zaehler,"-> LED RD%d");
    LATD = ledBit;
    DelayMs(1000);
    ledBit = ledBit << 1;
  } // for()

  LCD_Module_ClearLine(3);
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("-> Alle LEDs ein");
  LATD = 0xFF;
  DelayMs(1000);

  LCD_Module_ClearLine(3);
  LCD_Module_SetCursorPosition(1,3);
  LCD_Module_WriteString("-> Alle LEDs aus");
  LATD = 0x00;
  DelayMs(1000);

  LATD = 0x00;
  LCD_Module_ClearAll();
  
  TRISD  = 0xFF; // PortD[0-7]: Input-Port
}

// Funktion:     Menu_IOBox_In()
// Beschreibung: Test-Programm IO-Box Schalter
// Parameter:    -
// Rueckgabe:    -

void Menu_IOBox_In(void)
{
  bool flag_RD[8] = { false,false,false,false,false,false,false,false };
  unsigned int counter_RD[8] = {0};
  unsigned int durchlauf;
  unsigned int bitnr;

  LCD_Module_ClearAll();
  LCD_Module_SetCursorPosition(1, 1);
  LCD_Module_WriteString("16. IO-Box SW");

  LCD_Module_SetCursorPosition(1, 2);
  LCD_Module_WriteString("Port D Jumper");
  LCD_Module_SetCursorPosition(1, 3);
  LCD_Module_WriteString("auf Pos. Header");
  DelayMs(3000);
  LCD_Module_ClearLine(2);
  LCD_Module_ClearLine(3);

  TRISD = 0xFF; // PortD[0-7]: Input-Port

  LCD_Module_SetCursorPosition(1, 2);
  LCD_Module_WriteString("S7-6-5-4-3-2-1-0");
  for (durchlauf = 200; durchlauf > 0; durchlauf--)
  {
    LCD_Module_SetCursorPosition(14, 1);
    LCD_Module_WriteFormated_Int(durchlauf / 10, "%3d");
    
    for (bitnr = 0; bitnr < 8; bitnr++)
    {
      if ( (PORTD & (1<<bitnr)) != 0 )
      {
        LCD_Module_SetCursorPosition(15-(2*bitnr), 3);
        LCD_Module_WriteString("E");
        if (flag_RD[bitnr] == false)
        {
          flag_RD[bitnr] = true;
          counter_RD[bitnr]++;
          LCD_Module_SetCursorPosition(16-(2*bitnr), 3);
          LCD_Module_WriteFormated_Int(counter_RD[bitnr] % 10, "%1d");
        }
      }
      else
      {
        LCD_Module_SetCursorPosition(15-(2*bitnr), 3);
        LCD_Module_WriteString("A");
        if (flag_RD[bitnr] == true)
        {
          flag_RD[bitnr] = false;
          counter_RD[bitnr]++;
          LCD_Module_SetCursorPosition(16-(2*bitnr), 3);
          LCD_Module_WriteFormated_Int(counter_RD[bitnr] % 10, "%1d");
        }
      } // if
    } // for (bitnr = 0; bitnr < 8; bitnr++);
    
    DelayMs(100);
  } // for()

  TRISD = 0x00; // PortD[0-7]: Output-Port

  LCD_Module_ClearAll();
}

// Funktion:     timer1_isr()
// Beschreibung: Timer 1 Interrupt-Funktion  
// Parameter:    -
// Rueckgabe:    -
void timer1_isr(void)
{
  static bool led_flag = false;
  TMR1H |= 0x80;            // Set timer 1 to 0x8000 + current time
  timer1_zaehler++;         // Zaehler erhhen
  if (led_flag == true)
  {
    LED_GELB = 1;
    LED_ROT = 0;
    led_flag = false;
  }
  else
  {
    LED_GELB = 0;
    LED_ROT = 1;
    led_flag = true;
  }
}

// Funktion:     high_isr()
// Beschreibung: High priority interrupt Funktion
// Parameter:    -
// Rueckgabe:    -
void __interrupt() tc_int(void)
{
  // Timer 1 Interrupt ?
  if (TMR1IE && TMR1IF)
  {
    TMR1IF = 0;               // Clear timer 1 Interrupt
    timer1_isr();             // Timer 1 Interrupt-Funktion
  }
}

/*
// Funktion:     _high_ISR()
// Beschreibung: High priority interrupt Vektor-Tabelle
// Parameter:    -
// Rueckgabe:    -
#pragma code high_vector_section=0x08
void _high_ISR(void)
{
  _asm goto high_isr _endasm
}
#pragma code
*/
